/*
 * dht.c
 *
 *  Created on: 20/11/2013
 *      Author: Tomàs Clar
 */
#include "LPC17xx.h"
#include "CommonUtils.h"
#include "FreeRTOS.h"
#include "dht.h"
#include "print.h"
#include "task.h"

static uint8_t dht_read(uint8_t port, uint8_t * HumRel, uint8_t * Temp);

unsigned long ulPINState;

/*****************************************************************************
** Function name:		dht_init
**
** Descriptions:		Prepares pins P2.6, P2.7 and P2.8 for dht sensor
**
**
** parameters:			none
**
** Returned value:		None
**
*****************************************************************************/

void dht_init (void)
{
	//select GPIO function for P2.6 ~ P2.8
	LPC_PINCON->PINSEL4 &=(~(63<<12));
	LPC_GPIO2->FIODIR |=(7<<6);
	ulPINState = LPC_GPIO2->FIOPIN;
	LPC_GPIO2->FIOSET = ((~ ulPINState ) & (3<<6));

}
/*****************************************************************************
** Function name:		dht_request
**
** Descriptions:		Sends the start signal to dht sensor and wait for data
**
**
** parameters:			port, in-out parameters (Relative hum., Temperature)
**
** Returned value:		None
**
*****************************************************************************/
void dht_request(uint8_t dhtport, uint8_t * HumR, uint8_t * Temperature)
{

	vTaskSuspendAll();

	uint8_t Hrel, Temp, response;
	//start signal
	//pin off
	ulPINState = LPC_GPIO2->FIOPIN;
	LPC_GPIO2->FIOCLR = ulPINState & (1<<dhtport);
	activeDelay(20);
	//pin on
	ulPINState = LPC_GPIO2->FIOPIN;
	LPC_GPIO2->FIOSET = ((~ ulPINState ) & (1<<dhtport));
	activeDelay_us(30);
	//change direction
	LPC_GPIO2->FIODIR &= (~(1<<dhtport));
	//DHT sends out response signal (0) & keeps it for 80 us
	activeDelay_us(40);
	response=1;
	//watching for dht response
	if((LPC_GPIO2->FIOPIN &(1<<dhtport))==1)
	{
		response=0;
	}
	activeDelay_us(40);
	//DHT pulls up voltage and keeps it for 80 us
	activeDelay_us(45);
	if((LPC_GPIO2->FIOPIN &(1<<dhtport))==0)
	{
		response=0;
	}
	activeDelay_us(40);
	//start to transmit
	if(response != 0)
	{
		response=dht_read(dhtport, &Hrel, &Temp);
	}

	//change direction
	LPC_GPIO2->FIODIR |=(1<<dhtport);
	ulPINState = LPC_GPIO2->FIOPIN;
	LPC_GPIO2->FIOSET = ((~ ulPINState ) & (1<<dhtport));

	if (response ==0)
	{
		printText(portCP, "Error en sensor\r\n");
	}
	*HumR=Hrel;
	*Temperature = Temp;
	xTaskResumeAll();

}
/*****************************************************************************
** Function name:		dht_read
**
** Descriptions:		read the sequence sent by the dht sensor
**
**
** parameters:			port
**
** Returned value:		0 if check-sum is incorrect
** 						1 if check-sum is ok
**
*****************************************************************************/
uint8_t dht_read(uint8_t port, uint8_t * HumRel, uint8_t * Temp)
{
	uint8_t i, PINState = 0, HRint = 0, HRdec = 0, Tint = 0, Tdec = 0, resp = 0, sum = 0;

	//The first byte contains the integer part of RH
	for (i =0; i<8; i++)
	{
		while(!(LPC_GPIO2->FIOPIN &(1<<port)))
		{
		//	wait for high-voltage-level signal
		}
		activeDelay_us(40);
		PINState=((LPC_GPIO2->FIOPIN &(1<<port)) >> port);
		if (PINState == 1)
		{
			HRint |=(1<<(7-i));
			while(LPC_GPIO2->FIOPIN & (1<<port))
					{
					//wait for low-voltage-level signal
					}

		}
	}
	//The 2nd byte contains the decimal part of RH
		for (i =0; i<8; i++)
		{
			while(!(LPC_GPIO2->FIOPIN &(1<<port)))
			{
			//	wait for high-voltage-level signal
			}
			activeDelay_us(40);
			PINState=((LPC_GPIO2->FIOPIN &(1<<port)) >> port);
			if (PINState == 1)
			{
				HRdec |=(1<<(7-i));
				while(LPC_GPIO2->FIOPIN & (1<<port))
					{
					//wait for low-voltage-level signal
					}

		}
	}
	//The 3th byte contains the integral part of Temp
		for (i =0; i<8; i++)
		{
			while(!(LPC_GPIO2->FIOPIN &(1<<port)))
			{
			//	wait for high-voltage-level signal
			}
			activeDelay_us(40);
			PINState=((LPC_GPIO2->FIOPIN &(1<<port)) >> port);
			if (PINState == 1)
			{
				Tint |=(1<<(7-i));
				while(LPC_GPIO2->FIOPIN & (1<<port))
					{
						//wait for low-voltage-level signal
					}

		}
	}
	//The 4th byte contains the decimal part of Temp
	for (i =0; i<8; i++)
	{
		while(!(LPC_GPIO2->FIOPIN &(1<<port)))
		{
		//	wait for high-voltage-level signal
		}
		activeDelay_us(40);
		PINState=((LPC_GPIO2->FIOPIN &(1<<port)) >> port);
		if (PINState == 1)
		{
			Tdec |=(1<<(7-i));
			while(LPC_GPIO2->FIOPIN & (1<<port))
				{
				//wait for low-voltage-level signal
				}

		}
	}
	//The 5th byte contains the 8bit-check-sum
	for (i =0; i<8; i++)
	{
		while(!(LPC_GPIO2->FIOPIN &(1<<port)))
		{
		//	wait for high-voltage-level signal
		}
		activeDelay_us(40);
		PINState=((LPC_GPIO2->FIOPIN &(1<<port)) >> port);
		if (PINState == 1)
		{
			sum |=(1<<(7-i));
			while(LPC_GPIO2->FIOPIN & (1<<port))
				{
				//wait for low-voltage-level signal
				}

		}
	}
	*HumRel=HRint;
	*Temp=Tint;

	if((HRint + HRdec + Tint + Tdec)==sum)
	{
		resp = 1;
	}

	return resp;

}
